/***************************************************************************
 *   Copyright (C) 2008 by S. MANKOWSKI / G. DE BURE skrooge@miraks.com    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
#ifndef SKGRULEOBJECT_H
#define SKGRULEOBJECT_H
/** @file
 * This file is part of Skrooge and defines classes SKGRuleObject.
 *
 * @author Stephane MANKOWSKI / Guillaume DE BURE
 */

#include "skgdocument.h"
#include "skgobjectbase.h"
#include "skgerror.h"
#include "skgbankmodeler_export.h"

/**
 * This class allows to define rules
 */
class SKGBANKMODELER_EXPORT SKGRuleObject : public SKGObjectBase
{
        Q_OBJECT;

public:
        /**
         * Process mode
         */
        enum ProcessMode {ALL,
                          IMPORTED,
                          IMPORTEDNOTVALIDATE
                         };

        /**
         * Constructor
         * @param iDocument the document containing the object
         * @param iID the identifier in @p iTable of the object
         */
        explicit SKGRuleObject(const SKGDocument* iDocument=NULL, int iID=0);

        /**
         * Copy constructor
         * @param iObject the object to copy
         */
        SKGRuleObject(const SKGRuleObject& iObject);

        /**
         * Copy constructor
         * @param iObject the object to copy
         */
        SKGRuleObject(const SKGObjectBase& iObject);

        /**
         * Operator affectation
         * @param iObject the object to copy
         */
        virtual const SKGRuleObject& operator= (const SKGObjectBase &iObject);

        /**
         * Destructor
         */
        virtual ~SKGRuleObject();

        /**
         * Set the XML for the search definition
         * @param iXml the XML search definition
         * @return an object managing the error
         *   @see SKGError
         */
        virtual SKGError setXMLSearchDefinition(const QString& iXml);

        /**
         * Get the XML for the search definition
         * @return the XML
         */
        virtual QString getXMLSearchDefinition() const;

        /**
         * Set the XML for the action definition
         * @param iXml the XML action definition
         * @return an object managing the error
         *   @see SKGError
         */
        virtual SKGError setXMLActionDefinition(const QString& iXml);

        /**
         * Get the XML for the action definition
         * @return the XML
         */
        virtual QString getXMLActionDefinition() const;

        /**
         * Set the search description
         * @param iDescription the search description
         * @return an object managing the error
         *   @see SKGError
         */
        virtual SKGError setSearchDescription(const QString& iDescription);

        /**
         * Get the search description
         * @return the description
         */
        virtual QString getSearchDescription() const;

        /**
         * Set the action description
         * @param iDescription the action description
         * @return an object managing the error
         *   @see SKGError
         */
        virtual SKGError setActionDescription(const QString& iDescription);

        /**
         * Get the action description
         * @return the description
         */
        virtual QString getActionDescription() const;

        /**
         * Set the order for the rule
         * @param iOrder the order. (-1 means "at the end")
         * @return an object managing the error
         *   @see SKGError
         */
        virtual SKGError setOrder(double iOrder);

        /**
         * Get the order for the rule
         * @return the order
         */
        virtual double getOrder() const;

        /**
         * Execute actions
         * @param iMode mode
         * @return an object managing the error
         *   @see SKGError
         */
        virtual SKGError execute(const ProcessMode iMode);

        /**
         * Get where clause corresponding to to search condition
         * @param iAdditionalCondition additional select condition
         * @return
         */
        virtual QString getSelectSqlOrder(const QString& iAdditionalCondition="") const;

        /**
         * Get the list of supported operators
         * @param iAttributeType type of attribute
         * @param  iUpdateMode mode (true=update, false=select)
         * @return list of supported operators
         */
        static QStringList getListOfOperators(SKGServices::AttibuteType iAttributeType, bool iUpdateMode=false);

        /**
         * Get the NLS display of an operator
         * @param iOperator the operator (see @see getListOfOperators)
         * @param iParam1 parameter
         * @param iParam2 parameter
         * @return the NLS display
         */
        static QString getDisplayForOperator(const QString& iOperator, const QString& iParam1, const QString& iParam2, const QString& iAtt2);

protected:
        /**
         * Get the description of an XML definition
         * @param iDocument the document
         * @param iTable the table name
         * @param iXML the XML
         * @param iSQL to define if you want the TXT or SQL description
         * @param iMode how to generate:
         * 0 = for select order. Example: (t_comment='AB' and i_number=12) or (t_comment='hello')
         * 1 = for update order. Example: t_comment='AB', i_number=12, t_comment='hello'
         * 2 = for full update order. Example: update operation set t_comment='AB', i_number=12, t_comment='hello' where #WC#
         * @return the description
         */
        static QString getDescriptionFromXML(SKGDocument* iDocument, const QString& iTable, const QString& iXML, bool iSQL=false, int iMode=0);

private:
        static QStringList getFromXML(SKGDocument* iDocument, const QString& iTable, const QString& iXML, bool iSQL=false, int iMode=0);
};

#endif
